"""
明日方舟猜干员游戏插件
支持多种游戏模式和题库设置
"""

from .game_tools import *

# 插件元数据
__version__ = "0.3.1"
__author__ = "lizhiqi233-rgb"
__description__ = "明日方舟猜干员游戏 - 支持多种游戏模式和题库设置"

# 导出插件元数据
__all__ = ["__plugin_meta__"]
